@@ -12,16 +12,19 @@ test_that('abs_path() is working', {
 
 test_that('scientific notation with format_sci()',{
   options(digits = 4, scipen = 0)
-  expect_identical(format_sci(1), 1)
+  expect_identical(format_sci(1), '1')
   expect_identical(format_sci(0), '0')
-  expect_identical(format_sci(c(1.84e8, 1e5, 2.34e3)), c("$1.84\\times 10^{8}$", "$10^{5}$", "$2.34\\times 10^{3}$"))
-  expect_identical(format_sci(1.23456789*10^-5), "$1.2346\\times 10^{-5}$")
+  expect_identical(format_sci(c(1.84e8, 1e5, 2.34e3)),
+                   c(sprintf("\\ensuremath{%s}", c("1.84\\times 10^{8}", "10^{5}")), "2340"))
+  expect_identical(format_sci(1.23456789*10^-5), "\\ensuremath{1.2346\\times 10^{-5}}")
   expect_identical(format_sci(9.87654e6, 'html'), "9.8765 &times; 10<sup>6</sup>")
   expect_identical(format_sci(9.87654e6, 'rst'), "9.8765 |times| 10 :sup:`6`")
   expect_identical(format_sci(letters), letters)
+  expect_identical(format_sci(NA_real_), NA_character_)
 })
 
 test_that('fig_path() sanitizes paths', {
+  opts_knit$set(out.format = 'latex')
   expect_identical(fig_path('.png', list(fig.path = 'fig/', label = 'foo')), 'fig/foo.png')
   opts = list(fig.path = 'figure/', label = 'a b')
   expect_warning(fig_path(, opts))
@@ -32,6 +35,7 @@ test_that('fig_path() sanitizes paths', {
   )
   expect_identical(fig_path(, list(fig.path = '../', label = 'c.d')), '../c_d')
   expect_identical(fig_path(, list(fig.path = './../', label = 'c..d')), './../c__d')
+  opts_knit$set(out.format = NULL)
 })
 
 test_that('base64_encode() gets the same result as markdown:::.b64EncodeFile', {
